/* -*- C++ -*-
 *
 * File: "relatedlinks.js"
 * JavaScript source file for the Related Links sidebar
 *
 * Alex Eng
 * 2004-11-27
 */


const RELATEDLINKS_LOGGING_ENABLED = true;
const AWS_ALEXA_URL = "http://www.amazon.com/gp/browse.html/ref=sc_fe_c_1_3435361_7/102-0625751-1833753?%5Fencoding=UTF8&node=12782661&no=3435361&me=A36L942TSJ2AJA";

var gTabbedBrowser;
var gSidebar;
var gStrBundle;
var gClickedURL = null;
var gIsBusy = false;


function init() 
{
  gTabbedBrowser = parent.document.getElementById("content");
  gSidebar = document.getElementById("related-info-display");
  gStrBundle = document.getElementById("string-bundle");
  gIsBusy = true;

  updateCurrentURLDisplay();
  populateSidebar();
}


function updateCurrentURLDisplay() 
{
  var currentURL = getCurrentURL();
  var urlDisplay = document.getElementById("url-display");
  urlDisplay.value = "for " + currentURL;
  urlDisplay.tooltipText = currentURL;
}


function populateSidebar() 
{
  callAlexaService();
}


function callAlexaService()
{
  const METHOD_URI = "http://webservices.amazon.com/AWSAlexa/2004-09-15";
  const METHOD_NAME = "UrlInfo";

  var alexaService = new SOAPCall();
  alexaService.transportURI = "http://soap.amazon.com/onca/soap?Service=AlexaWebInfoService";

  var params = [];
  params[0] = new SOAPParameter(getAWSSubscriptionID(), "SubscriptionId");
  params[1] = new SOAPParameter("RelatedLinks,Rank,ContactInfo", "ResponseGroup");
  params[2] = new SOAPParameter(getCurrentURL(), "Url");
  alexaService.encode(0, METHOD_NAME, METHOD_URI, 0, null, params.length, params);

  toggleProgressIndicator();
  alexaService.asyncInvoke(function (response, alexaService, error) 
                           {
			     toggleProgressIndicator();
                             try {
                               handleResponse(response, error);
                             } catch (e) {
                               displayError(e);
			       gIsBusy = false;
                               return false;
                             }
                             processAlexaResults(response.body);
                           });
}


function handleResponse(response, error) 
{
  if (error != 0) {
    throw gStrBundle.getString("errorServiceFailure");
  }
  else {
    if (! response) {
      throw gStrBundle.getString("errorServiceUnavailable");
    }
    if (response.fault) {
      throw "Service fault: " + response.fault.faultString + " [Code: " + response.fault.faultCode + "]";
    }
  }
}


function processAlexaResults(resultDOM) 
{
  if (! resultDOM) {
    displayError("Error in processAlexaResults(): resultDOM cannot be null!");
    return false;
  }

  // Invalid AWS subscription ID or malformed SOAP request
  if (resultDOM.getElementsByTagName("IsValid").item(0).firstChild.data == "False") {
    displayError(gStrBundle.getString("errorInvalidRequest"));
    gIsBusy = false;
    return false;
  }

  var relatedLinks = resultDOM.getElementsByTagName("RelatedLink");
  if (relatedLinks.length == 0) {
    displayError(gStrBundle.getString("errorNoResults"));
    gIsBusy = false;
    return false;
  }

  listRelatedLinks(resultDOM);
  listRelatedInfo(resultDOM);
  listContactInfo(resultDOM);
  displayResultProvider();

  if (gIsBusy) {
    gIsBusy = false;
  }
}


function listRelatedLinks(resultDOM) 
{
  var relatedLinksNodeList = resultDOM.getElementsByTagName("RelatedLink");
  var titleNodeList = resultDOM.getElementsByTagName("Title");
  var navigableUrlNodeList = resultDOM.getElementsByTagName("NavigableUrl");
  if (! relatedLinksNodeList) {
    return;
  }

  var relatedLinksHdg = document.createElement("description");
  relatedLinksHdg.className = "hdg";
  relatedLinksHdg.setAttribute("value", "Related Links");
  gSidebar.appendChild(relatedLinksHdg);

  for (var i = 0; i < relatedLinksNodeList.length; i++) {
    var url = navigableUrlNodeList[i].firstChild.data;
    var anchorNode = document.createElement("description");
    anchorNode.className = "hyperlink";
    anchorNode.tooltipText = url;
    anchorNode.setAttribute("url", url);
    anchorNode.setAttribute("crop", "end");
    anchorNode.setAttribute("value", titleNodeList[i].firstChild.data);
    anchorNode.setAttribute("onclick", "relatedLink_click(event)");
    anchorNode.setAttribute("context", "hyperlink-context-menu");
    gSidebar.appendChild(anchorNode);
  }

  gSidebar.appendChild(document.createElement("separator"));
}


function relatedLink_click(event)
{
  var url = event.target.getAttribute("url");
  if (url) {
    gClickedURL = url;
  }

  // event.button values:
  //  0 - left mouse button
  //  2 - right mouse button
  if (event.button == 0) {
    gotoURL(url);
  }
}


function listRelatedInfo(resultDOM) 
{
  var rankNode = resultDOM.getElementsByTagName("Rank").item(0);
  if (! rankNode) {
    return;
  }

  var relatedInfoHdg = document.createElement("description");
  relatedInfoHdg.className = "hdg";
  relatedInfoHdg.setAttribute("value", "Related Site Info");
  gSidebar.appendChild(relatedInfoHdg);

  var trafficRanking = document.createElement("description");
  trafficRanking.setAttribute("value", "Traffic Ranking: " + rankNode.firstChild.data);
  gSidebar.appendChild(trafficRanking);
}


function listContactInfo(resultDOM) 
{
  var contactInfoNode = resultDOM.getElementsByTagName("ContactInfo").item(0);
  if (! contactInfoNode) {
    return;
  }

  // <OwnerName> multiplicity: [0..1]
  var ownerName = getNodeValue(contactInfoNode, "OwnerName");
  if (ownerName) {
    var ownerNameNode = document.createElement("description");
    ownerNameNode.className = "contact-info";
    ownerNameNode.appendChild(document.createTextNode(ownerName));
    gSidebar.appendChild(ownerNameNode);
  }

  // <Street> multiplicity: [1..*]
  var streetAddress = contactInfoNode.getElementsByTagName("Street");
  for (var i = 0; i < streetAddress.length; i++) {
    var streetNode = document.createElement("description");
    streetNode.className = "contact-info";
    streetNode.appendChild(document.createTextNode(streetAddress[i].firstChild.data));
    gSidebar.appendChild(streetNode);
  }

  // <City>, <State>, <PostalCode>, <Country>: each [0..1]
  var city = getNodeValue(contactInfoNode, "City");
  var state = getNodeValue(contactInfoNode, "State");
  var postalCode = getNodeValue(contactInfoNode, "PostalCode");
  var country = getNodeValue(contactInfoNode, "Country");

  if (city || state || postalCode) {
    var physLocNode = document.createElement("description");
    physLocNode.className = "contact-info";
    physLocNode.appendChild(document.createTextNode(city + ", " + state + "  " + postalCode));
    gSidebar.appendChild(physLocNode);
  }

  if (country) {
    var countryNode = document.createElement("description");
    countryNode.className = "contact-info";
    countryNode.appendChild(document.createTextNode(country));
    gSidebar.appendChild(countryNode);
  }

  // Only 1st <PhoneNumber> node retrieved; node multiplicity: [0..*]
  var phoneNumber = getNodeValue(contactInfoNode, "PhoneNumber");
  if (phoneNumber) {
    var phoneNumberNode = document.createElement("description");
    phoneNumberNode.className = "contact-info";
    phoneNumberNode.appendChild(document.createTextNode(phoneNumber));
    gSidebar.appendChild(phoneNumberNode);
  }
}


function getNodeValue(DOMNode, tagName) 
{
  var nodeList = DOMNode.getElementsByTagName(tagName);
  if (!nodeList || nodeList.length == 0) {
    return "";
  }

  var node = nodeList[0];
  if (!node.childNodes || node.childNodes.length == 0) {
    return "";
  }
  return (node.firstChild.data || "");
}


function displayResultProvider() 
{
  var descNode = document.createElement("description");
  var textNode = document.createTextNode("Related Links data provided by ");
  var linkNode = document.createElementNS("http://www.w3.org/1999/xhtml", "html:a");
  linkNode.className = "hyperlink";
  linkNode.setAttribute("href", "javascript:gotoURL('" + AWS_ALEXA_URL + "')");
  linkNode.appendChild(document.createTextNode("Alexa Web Information Service"));
  descNode.className = "result-provider";
  descNode.appendChild(textNode);
  descNode.appendChild(linkNode);  // Append <html:a> element
  
  gSidebar.appendChild(document.createElement("separator"));
  gSidebar.appendChild(descNode);
}


function getAWSSubscriptionID() 
{
  var cls = Components.classes["@mozilla.org/preferences-service;1"];
  var prefSvc = cls.getService(Components.interfaces.nsIPrefBranch);
  var subscriptionId = "";
  try {
    subscriptionId = prefSvc.getCharPref("relatedlinks.aws_subscription_id");
  } catch (e) { }

  return subscriptionId;
}


function reloadSidebar() 
{
  if (gIsBusy) {
    return;  // Don't interrupt sidebar reload in progress.
  }

  var childNodes = gSidebar.childNodes;
  gIsBusy = true;

  while (true) {
    var child = gSidebar.firstChild;
    try {
      gSidebar.removeChild(child);
    }
    catch (e if e.code == e.NOT_FOUND_ERR) {
      break;
    } 
    catch (e) {
      alert(e);
      break;
    }
  }

  updateCurrentURLDisplay();
    
  populateSidebar();
}


function toggleProgressIndicator()
{
  var progressMsg = document.getElementById("progress-msg");
  var progressBar = document.getElementById("progress-bar");
  if (progressMsg && progressBar) {
    gSidebar.removeChild(progressMsg);
    gSidebar.removeChild(progressBar);
  }
  else {
    progressMsg = document.createElement("description");
    progressBar = document.createElement("progressmeter");
    progressMsg.id = "progress-msg";
    progressBar.id = "progress-bar";
    progressMsg.setAttribute("value", gStrBundle.getString("msgPleaseWait"));
    progressMsg.style.fontWeight = "bold";
    progressBar.setAttribute("mode", "undetermined");
    gSidebar.appendChild(progressMsg);
    gSidebar.appendChild(progressBar);
  }
}


function showHelp()
{
  window.openDialog("chrome://relatedlinks/content/help.xul", "hlp", "modal");
}


function displayError(errorMsg) 
{
  var descNode = document.createElement("description");
  descNode.className = "error";
  descNode.style.color = "red";
  var textNode = document.createTextNode(errorMsg);
  descNode.appendChild(textNode);
  gSidebar.appendChild(descNode);
}


function getCurrentURL() 
{
  return gTabbedBrowser.selectedBrowser.currentURI.spec;
}


function gotoURL(url) 
{
  gTabbedBrowser.selectedBrowser.loadURI(url);
}


function openInNewWindow_command(event)
{
  window.open(gClickedURL);
}


function openInNewTab_command(event)
{
  gTabbedBrowser.addTab(gClickedURL);
}


function hideMenu_keypress(event)
{
  document.getElememtById("hyperlink-context-menu").hidePopup();
}



function log(aMessage) 
{
  if (RELATEDLINKS_LOGGING_ENABLED) {
    var consoleSvc = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
    consoleSvc.logStringMessage(aMessage);
  }
}
